@@ -8,7 +8,6 @@ from datetime import date, datetime
 from enum import Enum
 from typing import Any, Dict, Optional
 
-import dbt.exceptions
 import sqlparse
 import trino
 from dbt.adapters.base import Credentials
@@ -19,8 +18,6 @@ from dbt.exceptions import DatabaseException, FailedToConnectException, RuntimeE
 from dbt.helper_types import Port
 from trino.transaction import IsolationLevel
 
-from dbt.adapters.trino.__version__ import version
-
 logger = AdapterLogger("Trino")
 PREPARED_STATEMENTS_ENABLED_DEFAULT = True
 
@@ -146,6 +143,7 @@ class TrinoLdapCredentials(TrinoCredentials):
     port: Port
     user: str
     password: str
+    impersonation_user: Optional[str] = None
     roles: Optional[Dict[str, str]] = None
     cert: Optional[str] = None
     http_headers: Optional[Dict[str, str]] = None
@@ -277,7 +275,7 @@ class ConnectionWrapper(object):
         self._prepared_statements_enabled = prepared_statements_enabled
 
     def cursor(self):
-        self._cursor = self.handle.cursor()
+        self._cursor = self.handle.cursor(experimental_python_types=True)
         return self
 
     def cancel(self):
@@ -410,7 +408,9 @@ class TrinoConnectionManager(SQLConnectionManager):
         trino_conn = trino.dbapi.connect(
             host=credentials.host,
             port=credentials.port,
-            user=credentials.user,
+            user=credentials.impersonation_user
+            if getattr(credentials, "impersonation_user", None)
+            else credentials.user,
             roles=credentials.roles,
             catalog=credentials.database,
             schema=credentials.schema,
@@ -420,9 +420,9 @@ class TrinoConnectionManager(SQLConnectionManager):
             auth=credentials.trino_auth(),
             max_attempts=credentials.retries,
             isolation_level=IsolationLevel.AUTOCOMMIT,
-            source=f"dbt-trino-{version}",
+            source="dbt-trino",
+            verify=credentials.cert,
         )
-        trino_conn._http_session.verify = credentials.cert
         connection.state = "open"
         connection.handle = ConnectionWrapper(trino_conn, credentials.prepared_statements_enabled)
         return connection
@@ -462,11 +462,17 @@ class TrinoConnectionManager(SQLConnectionManager):
             )
 
         if cursor is None:
-            raise dbt.exceptions.RuntimeException(
+            conn = self.get_thread_connection()
+            if conn is None or conn.name is None:
+                conn_name = "<None>"
+            else:
+                conn_name = conn.name
+
+            raise RuntimeException(
                 "Tried to run an empty query on model '{}'. If you are "
                 "conditionally running\nsql, eg. in a model hook, make "
                 "sure your `else` clause contains valid sql!\n\n"
-                "Provided SQL:\n{}".format(connection.name, sql)
+                "Provided SQL:\n{}".format(conn_name, sql)
             )
 
         return connection, cursor
